@@ -54,7 +54,7 @@ class DownloadEngine;
 class AsyncNameResolverMan;
 #endif // ENABLE_ASYNC_DNS
 
-class DHTEntryPointNameResolveCommand:public Command {
+class DHTEntryPointNameResolveCommand : public Command {
 private:
   DownloadEngine* e_;
 
@@ -84,9 +84,9 @@ private:
 #endif // ENABLE_ASYNC_DNS
 
 public:
-  DHTEntryPointNameResolveCommand
-  (cuid_t cuid, DownloadEngine* e,
-   const std::vector<std::pair<std:: string, uint16_t> >& entryPoints);
+  DHTEntryPointNameResolveCommand(
+      cuid_t cuid, DownloadEngine* e,
+      const std::vector<std::pair<std::string, uint16_t>>& entryPoints);
 
   virtual ~DHTEntryPointNameResolveCommand();
 
